## Copyright (C) 2011 Uli Fouquet & Henrik Bettermann
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
## 
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
## 
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
##
"""
Tests for hostels. and their UI components
"""
import shutil
import tempfile
from zope.interface.verify import verifyClass, verifyObject
from zope.component.hooks import setSite, clearSite
from zope.testbrowser.testing import Browser
from zope.security.interfaces import Unauthorized
from waeup.sirp.app import University
from waeup.sirp.hostels.interfaces import (
    IHostelsContainer, IHostel, IBed)
from waeup.sirp.hostels.container import HostelsContainer
from waeup.sirp.hostels.hostel import Hostel, Bed
from waeup.sirp.testing import (FunctionalLayer, FunctionalTestCase)

class HostelsContainerTestCase(FunctionalTestCase):

    layer = FunctionalLayer

    def test_interfaces(self):
        # Make sure the correct interfaces are implemented.
        self.assertTrue(
            verifyClass(
                IHostelsContainer, HostelsContainer)
            )
        self.assertTrue(
            verifyObject(
                IHostelsContainer, HostelsContainer())
            )
        self.assertTrue(
            verifyClass(
                IHostel, Hostel)
            )
        self.assertTrue(
            verifyObject(
                IHostel, Hostel())
            )
        self.assertTrue(
            verifyClass(
                IBed, Bed)
            )
        self.assertTrue(
            verifyObject(
                IBed, Bed())
            )
        return

    def test_base(self):
        # We cannot call the fundamental methods of a base in that case
        container = HostelsContainer()
        self.assertRaises(
            NotImplementedError, container.archive)
        self.assertRaises(
            NotImplementedError, container.clear)

class HostelsFullSetup(FunctionalTestCase):

    def setUp(self):
        super(HostelsFullSetup, self).setUp()

        # Setup a sample site for each test
        app = University()
        self.dc_root = tempfile.mkdtemp()
        app['datacenter'].setStoragePath(self.dc_root)

        # Prepopulate the ZODB...
        self.getRootFolder()['app'] = app
        # we add the site immediately after creation to the
        # ZODB. Catalogs and other local utilities are not setup
        # before that step.
        self.app = self.getRootFolder()['app']
        # Set site here. Some of the following setup code might need
        # to access grok.getSite() and should get our new app then
        setSite(app)

        self.container_path = 'http://localhost/app/hostels'
        self.manage_container_path = self.container_path + '/@@manage'
        self.add_hostel_path = self.container_path + '/addhostel'

        # Put the prepopulated site into test ZODB and prepare test
        # browser
        self.browser = Browser()
        self.browser.handleErrors = False

    def tearDown(self):
        super(HostelsFullSetup, self).tearDown()
        clearSite()
        shutil.rmtree(self.dc_root)

class HostelsUITests(HostelsFullSetup):

    layer = FunctionalLayer

    def test_anonymous_access(self):
        # Anonymous users can't access hostels containers
        self.assertRaises(
            Unauthorized, self.browser.open, self.manage_container_path)
        return

    def test_add_search_edit_delete_hostels(self):
        self.browser.addHeader('Authorization', 'Basic mgr:mgrpw')
        self.browser.open(self.container_path)
        self.browser.getLink("Manage accommodation").click()
        self.assertEqual(self.browser.headers['Status'], '200 Ok')
        self.assertEqual(self.browser.url, self.manage_container_path)
        self.browser.getControl("Add hostel").click()
        self.assertEqual(self.browser.headers['Status'], '200 Ok')
        self.assertEqual(self.browser.url, self.add_hostel_path)
        self.browser.getControl("Create hostel").click()
        self.assertEqual(self.browser.headers['Status'], '200 Ok')
        self.assertTrue('Hostel created' in self.browser.contents)
        #import pdb; pdb.set_trace()
        hall = self.app['hostels']['hall_1']
        hall.blocks_for_female = ['A','B']
        self.browser.open(self.container_path + '/hall_1')
        expected = '''...<ul id="form.blocks_for_female" ><li>Block A</li>
<li>Block B</li></ul>...'''
        self.assertMatches(expected,self.browser.contents)
        self.browser.open(self.container_path + '/hall_1/edit')
        self.browser.getControl(name="form.nr_of_blocks").value = '5'
        self.browser.getControl("Save").click()
        self.assertTrue('Form has been saved' in self.browser.contents)
        self.browser.open(self.manage_container_path)
        ctrl = self.browser.getControl(name='val_id')
        value = ctrl.options[0]
        ctrl.getControl(value=value).selected = True
        self.browser.getControl("Remove selected", index=0).click()
        self.assertTrue('Successfully removed' in self.browser.contents)
